from django import template
from django.contrib.sites.shortcuts import get_current_site
from django.shortcuts import resolve_url

register = template.Library()


@register.simple_tag(takes_context=True)
def pageurl(context, page, fallback=None):
    request = context["request"]
    site = get_current_site(request)
    if page is None and fallback:
        return resolve_url(fallback)
    if not hasattr(page, "page_url"):
        raise ValueError("pageurl tag expected a Page object, got %r" % page)
    return page.page_url(site, request=context.get("request"))


@register.simple_tag(takes_context=True)
def routablepageurl(context, page, url_name, *args, **kwargs):
    # request = context["request"]
    # site = get_current_site(request)
    base_url = page.get_absolute_url()
    routed_url = page.reverse_subpage(url_name, args=args, kwargs=kwargs)
    if not base_url.endswith("/"):
        base_url += "/"
    return base_url + routed_url


@register.filter(name="proper_paginate")
def proper_paginate(paginator, current_page, neighbors=3):
    if paginator.num_pages > 2 * neighbors:
        start_index = max(1, current_page - neighbors)
        end_index = min(paginator.num_pages, current_page + neighbors)
        if end_index < start_index + 2 * neighbors:
            end_index = start_index + 2 * neighbors
        elif start_index > end_index - 2 * neighbors:
            start_index = end_index - 2 * neighbors
        if start_index < 1:
            end_index -= start_index
            start_index = 1
        elif end_index > paginator.num_pages:
            start_index -= end_index - paginator.num_pages
            end_index = paginator.num_pages
        page_list = [f for f in range(start_index, end_index + 1)]
        return page_list[: (2 * neighbors + 1)]
    return paginator.page_range


@register.simple_tag(takes_context=True)
def replace_param(context, **kwargs):
    """ """
    d = context["request"].GET.copy()
    for k, v in kwargs.items():
        d[k] = v
    for k in [k for k, v in d.items() if not v]:
        del d[k]
    return d.urlencode()


@register.simple_tag(takes_context=True)
def current_site(context):
    request = context["request"]
    site = get_current_site(request=request)
    return site


@register.simple_tag(takes_context=True)
def current_homepage(context):
    request = context["request"]
    site = get_current_site(request=request)
    try:
        return site.rootpage.page.get_absolute_url()
    except Exception:
        return "/"



# Generated by Django 3.2.12 on 2022-03-28 20:12

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import filer.fields.image
import mptt.fields
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('simpel_themes', '0004_alter_screenshoot_template'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=80, unique=True, verbose_name='Category Name')),
                ('slug', models.SlugField(blank=True, editable=False, max_length=80, null=True, unique=True)),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, help_text='Categories, unlike tags, can have a hierarchy. You might have a Jazz category, and under that have children categories for Bebop and Big Band. Totally optional.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='simpel_pages.category')),
            ],
            options={
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
                'ordering': ['name'],
                'permissions': (('import_category', 'Can import Category'), ('export_category', 'Can export Category')),
            },
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Last updated at')),
                ('url', models.CharField(db_index=True, max_length=100, verbose_name='URL')),
                ('title', models.CharField(max_length=200, verbose_name='title')),
                ('slug', models.SlugField(blank=True, editable=False, max_length=255, null=True, unique=True)),
                ('seo_title', models.CharField(blank=True, max_length=200, null=True, verbose_name='SEO title')),
                ('seo_description', models.TextField(blank=True, null=True, verbose_name='SEO description')),
                ('allow_comments', models.BooleanField(default=False, verbose_name='allow comments')),
                ('registration_required', models.BooleanField(default=False, help_text='If this is checked, only logged-in users will be able to view the page.', verbose_name='registration required')),
                ('data', models.JSONField(blank=True, null=True)),
                ('real_model', models.CharField(blank=True, editable=False, max_length=120, null=True)),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('bookmarks', models.ManyToManyField(blank=True, related_name='bookmarks', to=settings.AUTH_USER_MODEL, verbose_name='Users who bookmark this page.')),
                ('category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='pages', to='simpel_pages.category', verbose_name='Category')),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='pages', to=settings.AUTH_USER_MODEL, verbose_name='Page Owner')),
                ('parent', mptt.fields.TreeForeignKey(blank=True, help_text='Pages, unlike tags, can have a hierarchy. You might have a Index page, and under that have children pages for post and story. Totally optional.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='simpel_pages.page')),
                ('readers', models.ManyToManyField(blank=True, related_name='page_readers', to=settings.AUTH_USER_MODEL, verbose_name='Users who mark this page as read.')),
                ('sites', models.ManyToManyField(related_name='simpelposts', to='sites.Site', verbose_name='sites')),
            ],
            options={
                'verbose_name': 'page',
                'verbose_name_plural': 'pages',
                'ordering': ['url'],
            },
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='name')),
                ('slug', models.SlugField(max_length=100, unique=True, verbose_name='slug')),
            ],
            options={
                'verbose_name': 'Tag',
                'verbose_name_plural': 'Tags',
            },
        ),
        migrations.CreateModel(
            name='TaggedPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content_object', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tagged_pages', to='simpel_pages.page')),
                ('tag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tagged_pages', to='simpel_pages.tag')),
            ],
            options={
                'verbose_name': 'Tagged Page',
                'verbose_name_plural': 'Tagged Pages',
            },
        ),
        migrations.AddField(
            model_name='page',
            name='tags',
            field=taggit.managers.TaggableManager(blank=True, help_text='A comma-separated list of tags.', related_name='pages', through='simpel_pages.TaggedPage', to='simpel_pages.Tag', verbose_name='Tags'),
        ),
        migrations.AddField(
            model_name='page',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='pages', to='simpel_themes.modeltemplate', verbose_name='Template'),
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='simpel_pages.page')),
                ('content', models.TextField(blank=True, null=True, verbose_name='content')),
                ('summary', models.TextField(blank=True, null=True, verbose_name='summary')),
                ('thumbnail', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='simpelpages', to=settings.FILER_IMAGE_MODEL)),
            ],
            options={
                'verbose_name': 'post',
                'verbose_name_plural': 'posts',
                'abstract': False,
            },
            bases=('simpel_pages.page',),
        ),
        migrations.CreateModel(
            name='PageGallery',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('caption', models.CharField(max_length=200, verbose_name='Caption')),
                ('thumb_height', models.IntegerField(default=100, verbose_name='Thumbnail Height')),
                ('thumb_width', models.IntegerField(default=100, verbose_name='Thumbnail Width')),
                ('image', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='simpelpages_gallery_items', to=settings.FILER_IMAGE_MODEL)),
                ('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_query_name='galleries', to='simpel_pages.page', verbose_name='page')),
            ],
            options={
                'verbose_name': 'Image Gallery',
                'verbose_name_plural': 'Image Galleries',
                'ordering': ['page'],
                'index_together': {('page', 'image')},
            },
        ),
    ]



# Generated by Django 3.2.13 on 2022-04-13 06:24

from django.db import migrations, models
import tinymce.models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('simpel_pages', '0005_block_imageblock_richtextblock'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups this menu belongs to. A menu will get all permissions granted to each of their groups.', related_name='_simpel_pages_page_groups_+', to='auth.Group', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='category',
            name='description',
            field=tinymce.models.HTMLField(blank=True, null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='page',
            name='content',
            field=tinymce.models.HTMLField(blank=True, null=True, verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='richtextblock',
            name='text',
            field=tinymce.models.HTMLField(),
        ),
        migrations.AlterField(
            model_name='tag',
            name='description',
            field=tinymce.models.HTMLField(blank=True, null=True, verbose_name='Description'),
        ),
    ]



from django.apps import apps as django_apps
from django.contrib.sitemaps import Sitemap
from django.core.exceptions import ImproperlyConfigured


class PageSitemap(Sitemap):
    def items(self):
        if not django_apps.is_installed("django.contrib.sites"):
            raise ImproperlyConfigured(
                "SimpelPageSitemap requires django.contrib.sites, which isn't installed."
            )
        Site = django_apps.get_model("sites.Site")
        RootPage = django_apps.get_model("simpel_pages.RootPage", require_ready=False)
        current_site = Site.objects.get_current()
        root_page = RootPage.objects.get(site=current_site)
        return root_page.page.get_descendants().filter(registration_required=False, live=True)



# Generated by Django 3.2.12 on 2022-04-05 00:44

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import filer.fields.image


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
        ('simpel_pages', '0004_auto_20220403_2317'),
    ]

    operations = [
        migrations.CreateModel(
            name='Block',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.SlugField(blank=True, null=True, verbose_name='name')),
                ('group', models.SlugField(blank=True, null=True, verbose_name='group')),
                ('position', models.IntegerField(default=0, help_text='Used for ordering block.', verbose_name='position')),
                ('object_id', models.IntegerField(blank=True, null=True, verbose_name='object')),
                ('object_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='blocks', to='contenttypes.contenttype', verbose_name='object type')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_simpel_pages.block_set+', to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Block',
                'verbose_name_plural': 'Blocks',
            },
        ),
        migrations.CreateModel(
            name='RichTextBlock',
            fields=[
                ('block_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='simpel_pages.block')),
                ('text', models.TextField()),
            ],
            options={
                'verbose_name': 'Text Block',
                'verbose_name_plural': 'Text Blocks',
            },
            bases=('simpel_pages.block',),
        ),
        migrations.CreateModel(
            name='ImageBlock',
            fields=[
                ('block_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='simpel_pages.block')),
                ('caption', models.CharField(max_length=200, verbose_name='Caption')),
                ('thumb_height', models.IntegerField(default=100, verbose_name='Thumbnail Height')),
                ('thumb_width', models.IntegerField(default=100, verbose_name='Thumbnail Width')),
                ('image', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.FILER_IMAGE_MODEL)),
            ],
            options={
                'verbose_name': 'Image Block',
                'verbose_name_plural': 'Image Blocks',
            },
            bases=('simpel_pages.block',),
        ),
    ]



from django import template

register = template.Library()


@register.simple_tag(takes_context=True)
def render_block(context, block, **kwargs):
    request = context["request"]
    ctx = {"request": request}
    return block.render(request=request, context=ctx)



from django.conf import settings
from django.http import Http404
from django.utils.deprecation import MiddlewareMixin

from .views import simpelpage


class SimpelPageMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        if response.status_code != 404:
            return response  # No need to check for a simpelpage for non-404 responses.
        try:
            return simpelpage(request, request.path_info)
        # Return the original response if any errors happened. Because this
        # is a middleware, we can't assume the errors will be caught elsewhere.
        except Http404:
            return response
        except Exception:
            if settings.DEBUG:
                raise
            return response



from django import forms

from tinymce.widgets import TinyMCE

from .models import Page


class PageForm(forms.ModelForm):
    content = TinyMCE(
        attrs={"cols": 80, "rows": 20, "height": 97},
        mce_attrs={"height": 300},
    )

    class Meta:
        model = Page
        fields = "__all__"



from django.apps import AppConfig
from django.db.models.signals import post_migrate
from django.utils.translation import gettext_lazy as _


class SimpelPagesConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "simpel_pages"
    icon = "page-next-outline"
    label = "simpel_pages"
    verbose_name = _("Pages")

    def ready(self):
        post_migrate.connect(init_app, sender=self)


def init_app(**kwargs):
    pass



import datetime

from haystack import indexes

from .models import Page
from .settings import pages_settings


class PageIndex(indexes.SearchIndex, indexes.Indexable):

    text = indexes.CharField(
        document=True,
        use_template=True,
        template_name=pages_settings.PAGE_HAYSTACK_TEMPLATE,
    )
    author = indexes.CharField(model_attr="owner", null=True)
    created_at = indexes.DateTimeField(model_attr="created_at")

    def get_model(self):
        return Page

    def index_queryset(self, using=None):
        """Used when the entire index for model is updated."""
        return self.get_model().objects.filter(created_at__lte=datetime.datetime.now())



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _

from filer.fields.image import FilerImageField
from polymorphic.models import PolymorphicModel
from simpel_atomics.component import Component
from tinymce.models import HTMLField

VALID_RENDER_FORMAT = ["html", "dict", "json"]


class BlockComponent(Component):
    template_name = "simpel_pages/blocks/block.html"


class RichTextBlockComponent(Component):
    template_name = "simpel_pages/blocks/richtext.html"


class ImageBlockComponent(Component):
    template_name = "simpel_pages/blocks/image.html"


class BaseBlock(PolymorphicModel):
    name = models.SlugField(
        _("name"),
        null=True,
        blank=True,
    )
    group = models.SlugField(
        _("group"),
        null=True,
        blank=True,
    )
    position = models.IntegerField(
        default=0,
        verbose_name=_("position"),
        help_text=_("Used for ordering block."),
    )

    component_class = BlockComponent

    def get_component_class(self, request=None):
        return self.component_class

    def get_component_context(self, **kwargs):
        ctx = {"object": self}
        ctx.update(kwargs)
        return ctx

    def get_component_kwargs(self, **kwargs):
        return kwargs

    def get_component(self, request, **kwargs):
        return self.get_component_class(request)(**kwargs)

    def render(self, request=None, context=dict(), init_kwargs=dict()):
        ctx = self.get_component_context(**context)
        kwargs = self.get_component_kwargs(**init_kwargs)
        component = self.get_component(request, **kwargs)
        return component.render(ctx)

    class Meta:
        abstract = True

    def __str__(self):
        return "%s (%s)" % (self.__class__._meta.verbose_name, self.name)


class BlockManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().order_by("position")


class Block(BaseBlock):
    object_type = models.ForeignKey(
        ContentType,
        related_name="blocks",
        null=True,
        blank=True,
        verbose_name="object type",
        on_delete=models.CASCADE,
    )
    object_id = models.IntegerField(
        null=True,
        blank=True,
        verbose_name=_("object"),
    )
    object = GenericForeignKey(
        "object_type",
        "object_id",
    )

    class Meta:
        verbose_name = _("Block")
        verbose_name_plural = _("Blocks")


class RichTextBlock(Block):
    text = HTMLField()
    component_class = RichTextBlockComponent

    class Meta:
        verbose_name = _("Text Block")
        verbose_name_plural = _("Text Blocks")


class ImageBlock(Block):
    caption = models.CharField(
        max_length=200,
        verbose_name=_("Caption"),
    )
    thumb_height = models.IntegerField(
        default=100,
        verbose_name=_("Thumbnail Height"),
    )
    thumb_width = models.IntegerField(
        default=100,
        verbose_name=_("Thumbnail Width"),
    )
    image = FilerImageField(
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
    )
    component_class = ImageBlockComponent

    class Meta:
        verbose_name = _("Image Block")
        verbose_name_plural = _("Image Blocks")



import logging
import re

from django.conf import settings
from django.template.defaultfilters import slugify

import requests

logger = logging.getLogger(__name__)


def unique_slugify(instance, value, slug_field_name="slug", queryset=None, slug_separator="-"):
    """
    Calculates and stores a unique slug of ``value`` for an instance.

    ``slug_field_name`` should be a string matching the name of the field to
    store the slug in (and the field to check against for uniqueness).

    ``queryset`` usually doesn't need to be explicitly provided - it'll default
    to using the ``.all()`` queryset from the model's default manager.
    """
    slug_field = instance._meta.get_field(slug_field_name)

    slug = getattr(instance, slug_field.attname)
    slug_len = slug_field.max_length

    # Sort out the initial slug, limiting its length if necessary.
    slug = slugify(value)
    if slug_len:
        slug = slug[:slug_len]
    slug = _slug_strip(slug, slug_separator)
    original_slug = slug

    # Create the queryset if one wasn't explicitly provided and exclude the
    # current instance from the queryset.
    if queryset is None:
        queryset = instance.__class__._default_manager.all()
    if instance.pk:
        queryset = queryset.exclude(pk=instance.pk)

    # Find a unique slug. If one matches, at '-2' to the end and try again
    # (then '-3', etc).
    next = 2
    while not slug or queryset.filter(**{slug_field_name: slug}):
        slug = original_slug
        end = "%s%s" % (slug_separator, next)
        if slug_len and len(slug) + len(end) > slug_len:
            slug = slug[: slug_len - len(end)]
            slug = _slug_strip(slug, slug_separator)
        slug = "%s%s" % (slug, end)
        next += 1

    setattr(instance, slug_field.attname, slug)


def _slug_strip(value, separator="-"):
    """
    Cleans up a slug by removing slug separator characters that occur at the
    beginning or end of a slug.

    If an alternate separator is used, it will also replace any instances of
    the default '-' separator with the new separator.
    """
    separator = separator or ""
    if separator == "-" or not separator:
        re_sep = "-"
    else:
        re_sep = "(?:-|%s)" % re.escape(separator)
    # Remove multiple instances and if an alternate separator is provided,
    # replace the default '-' separator.
    if separator != re_sep:
        value = re.sub("%s+" % re_sep, separator, value)
    # Remove separator from the beginning and end of the slug.
    if separator:
        if separator != "-":
            re_sep = re.escape(separator)
        value = re.sub(r"^%s+|%s+$" % (re_sep, re_sep), "", value)
    return value


def get_ip(request):
    """
    Attempts to extract the IP number from the HTTP request headers.
    """
    key = "REMOTE_ADDR"
    meta = request.META

    # Lowercase keys
    simple_meta = {k.lower(): v for k, v in request.META.items()}
    ip = meta.get(key, simple_meta.get(key, "0.0.0.0"))
    return ip


def get_ip_location(ip):
    if settings.DEBUG:
        ip = "116.206.42.67"
    try:
        url = f"http://ip-api.com/json/{ip}"
        res = requests.get(
            url,
            headers={
                "Content-Type": "application/json",
                "User-Agent": "Mozilla/5.0",
            },
        )
        ip_data = res.json()
        ip_data.update(get_geo_location(ip_data["lat"], ip_data["lon"]))
        logger.info(ip_data)
        return ip_data
    except Exception as exc:
        logger.error(exc, exc_info=1)
        return None


def get_geo_location(lat: float, lon: float):
    res = requests.post(
        "https://geocode.arcgis.com/arcgis/rest/services/World/GeocodeServer/reverseGeocode",
        params={
            "f": "json",
            "location": f"{lon}, {lat}",
            "distance": 5000,
            "outSR": "",
        },
        headers={
            "Content-Type": "application/json",
        },
    )
    return res.json()



# Generated by Django 3.2.12 on 2022-04-02 18:57

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import filer.fields.image
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
        ('simpel_pages', '0002_auto_20220402_0304'),
    ]

    operations = [
        migrations.RenameField(
            model_name='page',
            old_name='text',
            new_name='content',
        ),
        migrations.AddField(
            model_name='page',
            name='thumbnail',
            field=filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='simpelpages', to=settings.FILER_IMAGE_MODEL),
        ),
        migrations.AddField(
            model_name='page',
            name='visitor_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='page',
            name='category',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='pages', to='simpel_pages.category', verbose_name='Category'),
        ),
        migrations.AlterField(
            model_name='page',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='Pages, unlike tags, can have a hierarchy. You might have a Index page, and under that have children pages for post and story. Totally optional.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='childs', to='simpel_pages.page'),
        ),
        migrations.CreateModel(
            name='Visitor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.CharField(max_length=16, null=True)),
                ('url', models.CharField(max_length=255)),
                ('path', models.CharField(max_length=255)),
                ('referrer', models.CharField(max_length=255, null=True)),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('content_id', models.CharField(max_length=255, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.contenttype')),
            ],
        ),
    ]



from django.http import Http404, HttpResponse
from django.template import loader
from django.template.response import TemplateResponse
from django.urls import URLResolver, re_path
from django.urls.resolvers import RegexPattern

_creation_counter = 0


def route(pattern, name=None):
    def decorator(view_func):
        global _creation_counter
        _creation_counter += 1

        # Make sure page has _routablepage_routes attribute
        if not hasattr(view_func, "_routablepage_routes"):
            view_func._routablepage_routes = []

        # Add new route to view
        view_func._routablepage_routes.append(
            (
                re_path(pattern, view_func, name=(name or view_func.__name__)),
                _creation_counter,
            )
        )

        return view_func

    return decorator


class RouteResult:
    """
    Taken from Wagtail !
    An object to be returned from Page.route, which encapsulates
    all the information necessary to serve an HTTP response. Analogous to
    django.urls.resolvers.ResolverMatch, except that it identifies
    a Page instance that we will call serve(*args, **kwargs) on, rather
    than a view function.
    """

    def __init__(self, page, args=None, kwargs=None):
        self.page = page
        self.args = args or []
        self.kwargs = kwargs or {}

    def __getitem__(self, index):
        return (self.page, self.args, self.kwargs)[index]


class RoutablePageMixin:
    """
    This class can be mixed in to a Page model, allowing extra routes to be
    added to it.
    """

    @route(r"^$")
    def index_route(self, request, *args, **kwargs):
        request.is_preview = getattr(request, "is_preview", False)

        return TemplateResponse(
            request,
            self.get_template(request, *args, **kwargs),
            self.get_context_data(request, **kwargs),
        )

    @classmethod
    def get_subpage_urls(cls):
        routes = []

        # Loop over this class's defined routes, in method resolution order.
        # Routes defined in the immediate class take precedence, followed by
        # immediate superclass and so on
        for klass in cls.__mro__:
            routes_for_class = []
            for val in klass.__dict__.values():
                if hasattr(val, "_routablepage_routes"):
                    routes_for_class.extend(val._routablepage_routes)

            # sort routes by _creation_counter so that ones earlier in the class definition
            # take precedence
            routes_for_class.sort(key=lambda route: route[1])

            routes.extend(route[0] for route in routes_for_class)

        return tuple(routes)

    @classmethod
    def get_resolver(cls):
        if "_routablepage_urlresolver" not in cls.__dict__:
            subpage_urls = cls.get_subpage_urls()
            cls._routablepage_urlresolver = URLResolver(RegexPattern(r"^/"), subpage_urls)

        return cls._routablepage_urlresolver

    def reverse_subpage(self, name, args=None, kwargs=None):
        """
        This method takes a route name/arguments and returns a URL path.
        """
        args = args or []
        kwargs = kwargs or {}

        return self.get_resolver().reverse(name, *args, **kwargs)

    def resolve_subpage(self, path):
        """
        This method takes a URL path and finds the view to call.
        """
        view, args, kwargs = self.get_resolver().resolve(path)

        # Bind the method
        view = view.__get__(self, type(self))

        return view, args, kwargs

    def route(self, request, path_components):
        """
        This hooks the subpage URLs into Wagtail's routing.
        """
        if self.live:
            try:
                path = "/"
                if path_components:
                    path += "/".join(path_components) + "/"

                view, args, kwargs = self.resolve_subpage(path)
                return RouteResult(self, args=(view, args, kwargs))
            except Http404:
                pass

        return super().route(request, path_components)

    def serve(self, request, view=None, args=None, kwargs=None):
        if args is None:
            args = []
        if kwargs is None:
            kwargs = {}
        if view is None:
            return super().serve(request, *args, **kwargs)
        return view(request, *args, **kwargs)

    def render(self, request, *args, template=None, context_overrides=None, **kwargs):
        if template is None:
            template = self.get_template(request, *args, **kwargs)
        else:
            template = loader.get_template(template)
        context = self.get_context_data(request, **kwargs)
        context.update(context_overrides or {})
        return HttpResponse(template.render(context, request))

    def serve_preview(self, request, mode_name):
        view, args, kwargs = self.resolve_subpage("/")
        request.is_preview = True
        request.preview_mode = mode_name

        return view(request, *args, **kwargs)



# Generated by Django 3.2.12 on 2022-04-02 03:04

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import filer.fields.image
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('simpel_themes', '0004_alter_screenshoot_template'),
        ('sites', '0002_alter_domain_unique'),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
        ('simpel_pages', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RootPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'verbose_name': 'Root',
                'verbose_name_plural': 'Roots',
                'swappable': 'SIMPEL_PAGE_SITE_MODEL',
            },
        ),
        migrations.RemoveField(
            model_name='post',
            name='page_ptr',
        ),
        migrations.RemoveField(
            model_name='post',
            name='thumbnail',
        ),
        migrations.AlterModelOptions(
            name='page',
            options={'ordering': ['-created_at']},
        ),
        migrations.RemoveField(
            model_name='page',
            name='sites',
        ),
        migrations.RemoveField(
            model_name='page',
            name='url',
        ),
        migrations.AddField(
            model_name='category',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='category',
            name='thumbnail',
            field=filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='pagecategories', to=settings.FILER_IMAGE_MODEL),
        ),
        migrations.AddField(
            model_name='page',
            name='live',
            field=models.BooleanField(default=True, verbose_name='live'),
        ),
        migrations.AddField(
            model_name='page',
            name='text',
            field=models.TextField(blank=True, null=True, verbose_name='Content'),
        ),
        migrations.AddField(
            model_name='page',
            name='url_path',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='page',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='Pages, unlike tags, can have a hierarchy. You might have a Index page, and under that have children pages for post and story. Totally optional.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='page_children', to='simpel_pages.page'),
        ),
        migrations.AlterField(
            model_name='page',
            name='slug',
            field=models.SlugField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='page',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='page_templates', to='simpel_themes.modeltemplate', verbose_name='Template'),
        ),
        migrations.AlterField(
            model_name='taggedpage',
            name='content_object',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tagged_posts', to='simpel_pages.page'),
        ),
        migrations.AlterField(
            model_name='taggedpage',
            name='tag',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tagged_posts', to='simpel_pages.tag'),
        ),
        migrations.DeleteModel(
            name='PageGallery',
        ),
        migrations.DeleteModel(
            name='Post',
        ),
        migrations.AddField(
            model_name='rootpage',
            name='page',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='site', to='simpel_pages.page', verbose_name='Page'),
        ),
        migrations.AddField(
            model_name='rootpage',
            name='site',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='rootpage', to='sites.site', verbose_name='Site'),
        ),
    ]



import logging

from django.apps import apps
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.contrib.auth.views import redirect_to_login
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.db import models
from django.http.response import Http404, HttpResponse
from django.template import loader
from django.urls import reverse
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView

from filer.fields.image import FilerImageField
from mptt.models import MPTTModel, TreeForeignKey
from simpel_themes.models import ModelTemplate
from taggit.managers import TaggableManager
from taggit.models import TagBase, TaggedItemBase
from tinymce.models import HTMLField

from simpel_pages.managers import CategoryManager, PageManager, TagManager
from simpel_pages.utils import unique_slugify

from ..routable import RouteResult
from ..settings import pages_settings

logger = logging.getLogger(__name__)


class IndexView(ListView):
    page_template = None

    def render_to_response(self, context, **response_kwargs):
        return HttpResponse(self.page_template.render(context, self.request))


class Category(MPTTModel):

    parent = TreeForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="children",
        help_text=_(
            "Categories, unlike tags, can have a hierarchy. You might have a "
            "Jazz category, and under that have children categories for Bebop"
            " and Big Band. Totally optional."
        ),
    )
    thumbnail = FilerImageField(
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="pagecategories",
    )
    name = models.CharField(
        max_length=80,
        unique=True,
        verbose_name=_("Category Name"),
    )
    description = HTMLField(
        null=True,
        blank=True,
        verbose_name=_("Description"),
    )
    slug = models.SlugField(
        unique=True,
        null=True,
        blank=True,
        editable=False,
        max_length=80,
    )
    created_at = models.DateTimeField(default=timezone.now)
    last_modified_at = models.DateTimeField(default=timezone.now)

    objects = CategoryManager()
    icon = "tag-outline"

    class Meta:
        ordering = ["name"]
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")
        permissions = (
            ("import_category", _("Can import Category")),
            ("export_category", _("Can export Category")),
        )

    def __str__(self):
        return self.name

    @property
    def opts(self):
        return self.__class__._meta

    def get_absolute_url(self):
        return reverse("serve_category", kwargs={"slug": self.slug})

    def clean(self):
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError("Parent category cannot be self.")
            if parent.parent and parent.parent == self:
                raise ValidationError("Cannot have circular Parents.")

    def save(self, *args, **kwargs):
        if not self.slug:
            unique_slugify(self, self.name)
        return super().save(*args, **kwargs)


class Tag(TagBase):

    description = HTMLField(
        null=True,
        blank=True,
        verbose_name=_("Description"),
    )
    created_at = models.DateTimeField(default=timezone.now)
    last_modified_at = models.DateTimeField(default=timezone.now)
    icon = "tag-outline"

    objects = TagManager()

    class Meta:
        verbose_name = _("Tag")
        verbose_name_plural = _("Tags")

    @property
    def opts(self):
        return self._meta

    def get_absolute_url(self):
        return reverse("serve_tag", kwargs={"slug": self.slug})


class Page(MPTTModel):
    created_at = models.DateTimeField(
        default=timezone.now,
        verbose_name=_("Created at"),
    )
    updated_at = models.DateTimeField(
        default=timezone.now,
        verbose_name=_("Last updated at"),
    )
    owner = models.ForeignKey(
        get_user_model(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="pages",
        verbose_name=_("Page Owner"),
    )
    parent = TreeForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="childs",
        help_text=_(
            "Pages, unlike tags, can have a hierarchy. You might have a "
            "Index page, and under that have children pages for post"
            " and story. Totally optional."
        ),
    )
    title = models.CharField(
        _("title"),
        max_length=200,
    )
    category = TreeForeignKey(
        Category,
        related_name="pages",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Category"),
    )
    tags = TaggableManager(
        through="TaggedPage",
        blank=True,
        related_name="pages",
        verbose_name=_("Tags"),
    )
    thumbnail = FilerImageField(
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="simpelpages",
    )
    content = HTMLField(
        null=True,
        blank=True,
        verbose_name=_("Content"),
    )
    data = models.JSONField(null=True, blank=True)
    slug = models.SlugField(
        unique=True,
        null=True,
        blank=True,
        db_index=True,
        max_length=255,
    )
    seo_title = models.CharField(
        _("SEO title"),
        null=True,
        blank=True,
        max_length=200,
    )
    seo_description = models.TextField(
        _("SEO description"),
        null=True,
        blank=True,
    )
    template = models.ForeignKey(
        ModelTemplate,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="%(class)s_templates",
        verbose_name=_("Template"),
    )
    url_path = models.CharField(
        max_length=255,
        null=True,
        blank=True,
    )
    index = models.BooleanField(
        _("index"),
        default=False,
        help_text=_("Index page, contain child pages and not included in widgets."),
    )
    per_page = models.IntegerField(
        default=0,
        verbose_name=_("per page"),
        help_text=_("Number item per page, used only when index is True."),
    )
    live = models.BooleanField(
        _("live"),
        default=True,
    )
    allow_comments = models.BooleanField(
        _("allow comments"),
        default=False,
    )
    registration_required = models.BooleanField(
        _("registration required"),
        help_text=_("If this is checked, only logged-in users will be able to view the page."),
        default=False,
    )
    groups = models.ManyToManyField(
        Group,
        verbose_name=_("groups"),
        blank=True,
        related_name="+",
        help_text=_(
            "The groups this menu belongs to. A menu will get all permissions " "granted to each of their groups."
        ),
    )
    real_model = models.CharField(
        max_length=120,
        editable=False,
        null=True,
        blank=True,
    )
    readers = models.ManyToManyField(
        get_user_model(),
        blank=True,
        related_name="page_readers",
        verbose_name=_("Users who mark this page as read."),
    )
    bookmarks = models.ManyToManyField(
        get_user_model(),
        blank=True,
        related_name="bookmarks",
        verbose_name=_("Users who bookmark this page."),
    )
    visitor_count = models.IntegerField(default=0)

    objects = PageManager()

    class Meta:
        ordering = ["-created_at"]

    def get_descendants(self, include_self=False):
        return super().get_descendants(include_self)

    def get_blocks(self):
        model = self.get_real_model_class()
        ctype = ContentType.objects.get_for_model(model)
        blocks = ctype.blocks.filter(object_id=self.id).order_by("position")
        return blocks

    def __str__(self):
        return "%s" % self.title

    @property
    def opts(self):
        return self.get_real_model_class()._meta

    @cached_property
    def blocks(self):
        return self.get_blocks()

    @property
    def specific(self):
        return self.get_real_instance()

    @property
    def page_type(self):
        return self.opts.verbose_name

    def has_group_perms(self, user=None):
        """Return True if user is superuser or in menu groups"""
        if user and getattr(user, "is_superuser", False):
            return True
        else:
            item_groups = {g.id for g in self.groups.all().values("id")}
            # Menu items not in groups
            if not bool(item_groups):
                return True
            user_groups = {g.id for g in user.groups.all().values("id")}
            permissions = user_groups.intersection(item_groups)
            return bool(permissions)

    def get_related_by_category(self, number=5):
        return self.__class__.objects.get_related_by_category(self)[:number]

    def get_related_by_tags(self, number=5):
        return self.__class__.objects.get_related_by_tags(self)[:number]

    def route(self, request, path_components):
        if path_components:
            # request is for a child of this page
            child_slug = path_components[0]
            remaining_components = path_components[1:]
            try:
                subpage = self.get_children().get(slug=child_slug)
            except self.__class__.DoesNotExist:
                raise Http404
            return subpage.specific.route(request, remaining_components)

        else:
            # request is for this very page
            if self.live:
                return RouteResult(self)
            else:
                raise Http404

    def get_context_data(self, request, **kwargs):
        context = {
            "object": self,
            "page_title": self.seo_title or self.title,
            "page_description": self.seo_description or self.title,
        }
        context.update(kwargs)
        return context

    def get_template(self, request, *args, **kwargs):
        if not self.template:
            template = loader.get_template(pages_settings.DEFAULT_TEMPLATE)
        else:
            template = self.template.specific
        return template

    def serve_index(self, request, *args, **kwargs):
        context = self.get_context_data(request, **kwargs)
        constructor = {
            "paginate_by": self.per_page or pages_settings.ITEM_PER_PAGE,
            "page_template": self.get_template(request),
            "extra_context": context,
            "queryset": self.childs.live(),
            "response_class": HttpResponse,
        }
        return IndexView.as_view(**constructor)(request, *args, **kwargs)

    def serve_page(self, request, *args, **kwargs):
        template = self.get_template(request, *args, **kwargs)
        context = self.get_context_data(request, **kwargs)
        return HttpResponse(template.render(context, request))

    def serve(self, request, *args, **kwargs):
        #  Check visitor has permissions
        if self.groups.first():
            if not request.user.is_authenticated:
                return redirect_to_login(request.path)
            else:
                if not self.has_group_perms(request.user):
                    raise PermissionError(_("You don't have any permission to view this page."))
        #  Make visitor is registered
        if self.registration_required and not request.user.is_authenticated:
            return redirect_to_login(request.path)
        if self.index:
            return self.serve_index(request, *args, **kwargs)
        return self.serve_page(request, *args, **kwargs)

    def set_url_path(self):
        if self.parent is not None:
            self.url_path = self.parent.url_path + self.slug + "/"
        else:
            self.url_path = ""
        return self.url_path

    def page_url(self, site=None, request=None):
        return reverse("serve_pages", args=(self.url_path,))

    def get_absolute_url(self, site=None, request=None):
        return self.page_url(site, request)

    def get_model_name(self):
        return "%s.%s" % (self.opts.app_label, self.opts.model_name)

    def get_real_model_class(self):
        """
        Return the real Model class related to objects.
        """
        try:
            return apps.get_model(self.real_model, require_ready=False)
        except Exception:
            if self.real_model is not None:
                logger.info("real model refers to model '%s' that has not been installed" % self.real_model)
            return self.__class__

    def get_real_instance(self):
        """Return the real page instance."""
        model = self.get_real_model_class()
        if model.__name__ == self.__class__.__name__:
            return self
        instance = model.objects.get(pk=self.id)
        return instance

    def get_root(self):
        root = super().get_root()
        instance = root.get_real_instance()
        return instance

    def clean(self):
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError("Parent page cannot be self.")
            if parent.parent and parent.parent == self:
                raise ValidationError("Cannot have circular Parents.")

    def save(self, *args, **kwargs):
        self.updated_at = timezone.now()
        if not self.slug:
            unique_slugify(self, self.title)
        if not self.real_model:
            self.real_model = self.get_model_name()
        self.set_url_path()
        return super().save(*args, **kwargs)


class RootPage(models.Model):
    site = models.OneToOneField(
        Site,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="rootpage",
        verbose_name=_("Site"),
    )
    page = models.OneToOneField(
        Page,
        on_delete=models.SET_NULL,
        null=True,
        related_name="site",
        blank=True,
        verbose_name=_("Page"),
    )

    class Meta:
        verbose_name = _("Root")
        verbose_name_plural = _("Roots")
        swappable = "SIMPEL_PAGE_SITE_MODEL"

    def __str__(self):
        return "%s root Page" % self.site


class TaggedPage(TaggedItemBase):

    content_object = models.ForeignKey(
        Page,
        on_delete=models.CASCADE,
        related_name="tagged_posts",
        db_index=True,
    )
    tag = models.ForeignKey(
        Tag,
        on_delete=models.CASCADE,
        related_name="tagged_pages",
        db_index=True,
    )

    class Meta:
        verbose_name = _("Tagged Page")
        verbose_name_plural = _("Tagged Pages")

    def __str__(self):
        return str(self.tag)


class Visitor(models.Model):
    ip = models.CharField(max_length=16, null=True)
    url = models.CharField(max_length=255)
    path = models.CharField(max_length=255)
    referrer = models.CharField(max_length=255, null=True)
    created_at = models.DateTimeField(default=timezone.now)
    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    content_id = models.CharField(max_length=255, null=True)
    content = GenericForeignKey("content_type", "content_id")



# Generated by Django 3.2.12 on 2022-04-03 23:17

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('simpel_pages', '0003_auto_20220402_1857'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='category',
            name='last_modified_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='page',
            name='index',
            field=models.BooleanField(default=False, help_text='Index page, contain child pages and not included in widgets.', verbose_name='index'),
        ),
        migrations.AddField(
            model_name='page',
            name='per_page',
            field=models.IntegerField(default=0, help_text='Number item per page, used only when index is True.', verbose_name='per page'),
        ),
        migrations.AddField(
            model_name='tag',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='tag',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='tag',
            name='last_modified_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='page',
            name='url_path',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='taggedpage',
            name='tag',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tagged_pages', to='simpel_pages.tag'),
        ),
    ]



from django.contrib.sites.shortcuts import get_current_site
from django.core.management.base import BaseCommand

from simpel_themes.models import PathModelTemplate

from simpel_pages.models import Page, RootPage


class Command(BaseCommand):
    help = "Install demo pages"

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        home_template, _ = PathModelTemplate.objects.get_or_create(
            template="simpel_pages/home.html",
            defaults={
                "name": "Page Home",
            },
        )
        index_template, _ = PathModelTemplate.objects.get_or_create(
            template="simpel_pages/index.html",
            defaults={
                "name": "Page Index",
            },
        )
        detail_template, _ = PathModelTemplate.objects.get_or_create(
            template="simpel_pages/detail.html",
            defaults={
                "name": "Page Detail",
            },
        )

        site = get_current_site(request=None)
        root_page = getattr(site, "rootpage", None)
        if root_page is None:
            root_page, _ = RootPage.objects.get_or_create(site=site)

        if root_page.page is None:
            home_page, _ = Page.objects.get_or_create(
                parent=None,
                slug="home",
                defaults={
                    "template": home_template,
                    "title": "Home Page",
                    "slug": "home",
                    "index": True,
                },
            )
            root_page.page = home_page
            root_page.save()
        else:
            home_page = root_page.page

        blog_page, _ = Page.objects.get_or_create(
            parent=home_page,
            slug="blog",
            defaults={
                "template": index_template,
                "title": "Blog",
                # "slug": "blog",
                "index": True,
            },
        )
        post_1, _ = Page.objects.get_or_create(
            parent=blog_page,
            slug="post-1",
            defaults={
                "template": detail_template,
                "title": "Post 1",
                # "slug": "post-1",
            },
        )
        post_2, _ = Page.objects.get_or_create(
            parent=blog_page,
            slug="post-2",
            defaults={
                "template": detail_template,
                "title": "Post 2",
                # "slug": "post-2",
            },
        )
        post_3, _ = Page.objects.get_or_create(
            parent=blog_page,
            slug="post-3",
            defaults={
                "template": detail_template,
                "title": "Post 3",
                # "slug": "post-3",
            },
        )
        toc_page, _ = Page.objects.get_or_create(
            parent=home_page,
            slug="term-of-condition",
            defaults={
                "index": True,
                "title": "Term of Condition",
                # "slug": "term-of-condition",
            },
        )
        privacy_page, _ = Page.objects.get_or_create(
            parent=home_page,
            slug="privacy-policy",
            defaults={
                "index": True,
                "title": "Privacy Policy",
                # "slug": "privacy-policy",
            },
        )
